/*
 * CruiseList.java
 *
 * Created on January 14, 2002, 4:02 PM
 */

package unleashed.ch2;

import javax.swing.*;
import java.awt.event.ActionListener;
import javax.swing.border.EtchedBorder;
import java.awt.Container;
import java.awt.BorderLayout;
import java.sql.*;

import java.rmi.*;
import javax.rmi.*;
import java.util.*;
import java.io.*;

/**
 *
 * @author  Stephen Potts
 * @version
 */
public class CruiseList extends JFrame implements ActionListener
{
    //RMI variables
    static TicketAgent ta;
    
    //JDBC variables
    java.sql.Connection dbConn = null;
    Statement statement1 = null;
       
    String server;
    String user;
    String password;
        
    //GUI Variables
    JList customerList;
    JList cruisesAvailable;
    JButton btnBook;
    JButton btnCheck;
    JButton btnExit;
    
    //Arrays to hold database information
    int[] custNums = null;
    String[] lastNames = null;
    String[] firstNames = null;
    
    int[] cruiseIDs = null;
    String[] cruiseDestinations = null;
    String[] cruisePorts = null;
    String[] cruiseSailings = null;
    
    /** Constructors for CruiseList */
    public CruiseList() throws Exception
    {
        init();
    }
    
    public CruiseList(String caption) throws Exception
    {
        super(caption);
        init();
    }
    
    //The init() method moves real processing out
    //of the constructors where
    //Exception handling is simpler
    private void init() throws Exception
    {
        try
        {
            //Obtain connections to JDBC 
            ConnectToServices();
            
            //configure the Frame
            setBounds(150,200,500,250);
            setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            
            //set the layout
            BorderLayout border = new BorderLayout();
            Container content = getContentPane();
            content.setLayout(border);
            
            //Populate the customerNames string
            String getCustomerString =
            "SELECT CustomerID, LastName, FirstName FROM CruiseCustomer";
            ResultSet custResults =
                         statement1.executeQuery(getCustomerString);
            
            custNums = new int[20];
            lastNames = new String[20];
            firstNames = new String[20];
            int index = 0;
            
            while (custResults.next())
            {
                custNums[index] = custResults.getInt("CustomerID");
                firstNames[index] = custResults.getString("FirstName");
                lastNames[index] = custResults.getString("LastName");
                index += 1;
            }
            
            String[] customerNames = new String[index];
            
            for (int i=0;i<index;i++)
            {
                customerNames[i] = firstNames[i] + " " + lastNames[i];
            }
            int numCustomers = index;
            System.out.println("The number of customers " + index);
            
            //Populate the cruises string
            String getCruiseString =
            "SELECT CruiseID, Destination, Port, Sailing FROM Cruises";
            ResultSet cruiseResults =
                     statement1.executeQuery(getCruiseString);
            
            cruiseIDs = new int[20];
            cruiseDestinations = new String[20];
            cruisePorts = new String[20];
            cruiseSailings = new String[20];
            index = 0;
            
            while (cruiseResults.next())
            {
                cruiseIDs[index] = cruiseResults.getInt("CruiseID");
                cruiseDestinations[index] = 
                     cruiseResults.getString("Destination");
                cruisePorts[index] = cruiseResults.getString("Port");
                cruiseSailings[index] = 
                          cruiseResults.getString("Sailing");
                index += 1;
            }
            
            String[] cruises = new String[index];
            
            for (int i=0;i<index;i++)
            {
                cruises[i] = cruiseDestinations[i] +
                "      Departs: " + cruisePorts[i] +
                " " + cruiseSailings[i];
            }
            
            int numCruises = index;
            System.out.println("The number of cruises" + index);
            
            
            //More GUI components
            String labelString = "               Customer";
            labelString += "                          Cruise";
            JLabel label1 = new JLabel(labelString);
            customerList = new JList(customerNames);
            cruisesAvailable = new JList(cruises);
            btnBook = new JButton("Book");
            btnCheck = new JButton("Check");
            btnExit = new JButton("Exit");
            btnBook.addActionListener(this);
            btnCheck.addActionListener(this);
            btnExit.addActionListener(this);
            
            JPanel bottomPanel = new JPanel();
            JPanel centerPanel = new JPanel();
            centerPanel.add(new JScrollPane(customerList, 
                  JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
            centerPanel.add(new JScrollPane(cruisesAvailable,
            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
            bottomPanel.add(btnBook);
            bottomPanel.add(btnCheck);
            bottomPanel.add(btnExit);
            content.add(label1, BorderLayout.NORTH);
            content.add(centerPanel, BorderLayout.CENTER);
            content.add(bottomPanel, BorderLayout.SOUTH);
            setVisible(true);
        }catch(Exception e)
        {
            System.out.println("Exception thrown " + e);
        } finally
        {
            try
            {
                //close all connection
                if (statement1 != null)
                    statement1.close();
                if (dbConn != null)
                    dbConn.close();
                
            } catch (SQLException sqle)
            {
                System.out.println(
                       "SQLException during close(): " +
                                      sqle.getMessage());
            }
        }
        
    }
    
    private void ConnectToServices()
    {
        try
        {
            
            //Get a connection to the datasource
            //load the driver class
            Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
            
            //Specify the ODBC data source
            String sourceURL = "jdbc:odbc:CruiseTicket";
            
            //get a connection to the database
            dbConn =DriverManager.getConnection(sourceURL);
            
            //If we get to here, no exception was thrown
            System.out.println("The database connection is " + dbConn);
            System.out.println("Making connection...\n");
            
            //Create the statement
            statement1 = dbConn.createStatement();
            
            
        } catch (Exception e)
        {
            System.out.println("Exception was thrown: " + e);
        }
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[])
    {
        try
        {
            //create a handle to the TicketAgentService
            ta = (TicketAgent)Naming.lookup(
                    "rmi://localhost:1099/TicketAgentService");
            
            //create an instance of the GUI
            CruiseList mainWindow =
                  new CruiseList("Cruise Ticket System");
        }catch(Exception e)
        {
            System.out.println("Exception in main " + e);
        }
    }
    
    public void actionPerformed(java.awt.event.ActionEvent ae)
    {
        try
        {
            TicketRequest tickReq2 = null;
            Container c = btnExit.getParent();
            if (ae.getActionCommand().equals("Exit"))
            {
                System.exit(0);
            }
            
            //Try and book a ticket
            if (ae.getActionCommand().equals("Book"))
            {
                System.out.println("Book was clicked");
                int custIndex = customerList.getSelectedIndex();
                int cruiseIndex =
                        cruisesAvailable.getSelectedIndex();
                
                if (custIndex == -1 || cruiseIndex == -1)
                {
                    JOptionPane.showMessageDialog(c,
                    "You must choose a customer and a cruise");
                }else
                {
                    //Pop up a dialog asking how many tickets
                    //are requested
                    String numTickets = JOptionPane.showInputDialog(c,
                    "How many tickets?");
                    int numberOfTickets = Integer.parseInt(numTickets);
                    
                    //create a ticket request object
                    TicketRequest tickReq = 
                               new TicketRequest(custNums[custIndex],
                    lastNames[custIndex], firstNames[custIndex],
                    cruiseIDs[cruiseIndex], 
                    cruiseDestinations[cruiseIndex],
                    cruisePorts[cruiseIndex], 
                    cruiseSailings[cruiseIndex],
                    numberOfTickets);
                    
                    System.out.println(tickReq);
                    //Create a message using this object
                    
                    try
                    {
                        Object obj2 = ta.createTicket(tickReq);
                        System.out.println("TicketRequest sent via RMI");
                        
                        tickReq2 = (TicketRequest)obj2;
                        System.out.println(tickReq2);
                        
                    }catch (Exception e)
                    {
                        log("Error making RMI call " + e);
                    }
                    
                    System.out.println("Ticket Request complete ");
                    if (tickReq2.isApproved())
                    {
                        displayResult("The ticket for " +
                                       tickReq2.getLastName() +
                                      " has been approved");
                    }else
                    {
                        displayResult("The ticket for " +
                                       tickReq2.getLastName() +
                                      " was not approved");
                    }
                }
            }
            
        }catch (Exception e)
        {
            System.out.println("Exception thrown = " + e);
        }
    }
       
    private void log(String s)
    {
        System.out.println(s);
    }
    
    public void displayResult(String str)
    {
        try
        {
            Container c = btnExit.getParent();
            
            //show the message in a dialog box
            JOptionPane.showMessageDialog(c, str);
        }catch (Exception e)
        {
            System.out.println("o Exception " + e);
        }
    }
}