package unleashed.ch3;

/*
 * CruiseList.java
 *
 * Created on December 31, 2001, 6:35 PM
 */

import unleashed.TicketRequest2;
import javax.swing.*;
import java.awt.event.ActionListener;
import javax.swing.border.EtchedBorder;
import java.awt.Container;
import java.awt.BorderLayout;
import java.sql.*;
import javax.naming.*;
import java.util.*;

import java.io.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import javax.xml.parsers.*;


import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

/**
 *
 * @author  Stephen Potts
 * @version
 */
public class CruiseList extends JFrame implements ActionListener
{
    Document doc;
    
    //JDBC variables
    java.sql.Connection dbConn = null;
    Statement statement1 = null;
    
    //GUI Variables
    JList customerList;
    JList cruisesAvailable;
    JButton btnBook;
    JButton btnExit;
    JButton btnCheck;
    
    
    //Arrays to hold database information
    int[] custNums = null;
    String[] lastNames = null;
    String[] firstNames = null;
    
    int[] cruiseIDs = null;
    String[] cruiseDestinations = null;
    String[] cruisePorts = null;
    String[] cruiseSailings = null;
    
    /** Constructors for CruiseList */
    public CruiseList() throws Exception
    {
        init();
    }
    
    public CruiseList(String caption) throws Exception
    {
        super(caption);
        init();
    }
    
    //The init() method moves processing out of the constructors where
    //Exception handling is simpler
    private void init() throws Exception
    {
        try
        {
            //Obtain connections to JDBC and JMS via JNDI
            ConnectToServices();
            
            //configure the Frame
            setBounds(150,200,500,250);
            setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            
            //set the layout
            BorderLayout border = new BorderLayout();
            Container content = getContentPane();
            content.setLayout(border);
            
            //Populate the customerNames string
            String getCustomerString =
            "SELECT CustomerID, LastName, FirstName FROM CruiseCustomer";
            ResultSet custResults =
            statement1.executeQuery(getCustomerString);
            
            custNums = new int[20];
            lastNames = new String[20];
            firstNames = new String[20];
            int index = 0;
            
            while (custResults.next())
            {
                custNums[index] = custResults.getInt("CustomerID");
                firstNames[index] = custResults.getString("FirstName");
                lastNames[index] = custResults.getString("LastName");
                index += 1;
            }
            
            String[] customerNames = new String[index];
            
            for (int i=0;i<index;i++)
            {
                customerNames[i] = firstNames[i] + " " + lastNames[i];
            }
            int numCustomers = index;
            System.out.println("The number of customers " + index);
            
            //Populate the cruises string
            String getCruiseString =
            "SELECT CruiseID, Destination, Port, Sailing FROM Cruises";
            ResultSet cruiseResults =
            statement1.executeQuery(getCruiseString);
            
            cruiseIDs = new int[20];
            cruiseDestinations = new String[20];
            cruisePorts = new String[20];
            cruiseSailings = new String[20];
            index = 0;
            
            while (cruiseResults.next())
            {
                cruiseIDs[index] = cruiseResults.getInt("CruiseID");
                cruiseDestinations[index] =
                cruiseResults.getString("Destination");
                cruisePorts[index] = cruiseResults.getString("Port");
                cruiseSailings[index] =
                cruiseResults.getString("Sailing");
                index += 1;
            }
            
            String[] cruises = new String[index];
            
            for (int i=0;i<index;i++)
            {
                cruises[i] = cruiseDestinations[i] +
                "      Departs: " +
                cruisePorts[i] + " " + cruiseSailings[i];
            }
            
            int numCruises = index;
            System.out.println("The number of cruises" + index);
            
            
            //More GUI components
            String labelString = "               Customer";
            labelString += "                          Cruise";
            JLabel label1 = new JLabel(labelString);
            customerList = new JList(customerNames);
            cruisesAvailable = new JList(cruises);
            btnBook = new JButton("Book");
            btnCheck = new JButton("Check");
            btnExit = new JButton("Exit");
            btnBook.addActionListener(this);
            btnExit.addActionListener(this);
            btnCheck.addActionListener(this);
            
            JPanel bottomPanel = new JPanel();
            JPanel centerPanel = new JPanel();
            centerPanel.add(new JScrollPane(customerList,
            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
            centerPanel.add(new JScrollPane(cruisesAvailable,
            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
            bottomPanel.add(btnBook);
            bottomPanel.add(btnCheck);
            bottomPanel.add(btnExit);
            content.add(label1, BorderLayout.NORTH);
            content.add(centerPanel, BorderLayout.CENTER);
            content.add(bottomPanel, BorderLayout.SOUTH);
            setVisible(true);
        }catch(Exception e)
        {
            System.out.println("Exception thrown " + e);
        } finally
        {
            try
            {
                //close all connection
                if (statement1 != null)
                    statement1.close();
                if (dbConn != null)
                    dbConn.close();
                
            } catch (SQLException sqle)
            {
                System.out.println("SQLException during close(): " +
                sqle.getMessage());
            }
        }
        
    }
    
    private void ConnectToServices()
    {
        try
        {
            // ============== Make connection to database ============
            
            //load the driver class
            Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
            
            //Specify the ODBC data source
            String sourceURL = "jdbc:odbc:CruiseTicket";
            
            //get a connection to the database
            dbConn =DriverManager.getConnection(sourceURL);
            
            //If we get to here, no exception was thrown
            System.out.println("The database connection is " + dbConn);
            System.out.println("Making connection...\n");
            
            //Create the statement
            statement1 = dbConn.createStatement();
            
            
        } catch (Exception e)
        {
            System.out.println("Exception was thrown: " + e);
        }
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[])
    {
        //create an instance of the GUI
        try
        {
            CruiseList mainWindow =
            new CruiseList("Cruise Ticket System");
        }catch(Exception e)
        {
            System.out.println("Exception in main " + e);
        }
    }
    
    public void actionPerformed(java.awt.event.ActionEvent ae)
    {
        try
        {
            Container c = btnExit.getParent();
            if (ae.getActionCommand().equals("Exit"))
            {
                System.exit(0);
            }
            
            //Try and book a ticket
            if (ae.getActionCommand().equals("Book"))
            {
                System.out.println("Book was clicked");
                int custIndex = customerList.getSelectedIndex();
                int cruiseIndex = cruisesAvailable.getSelectedIndex();
                
                if (custIndex == -1 || cruiseIndex == -1)
                {
                    JOptionPane.showMessageDialog(c,
                    "You must choose a customer and a cruise");
                }else
                {
                    //Pop up a dialog asking how many tickets
                    String numTickets = JOptionPane.showInputDialog(c,
                    "How many tickets?");
                    int numberOfTickets = Integer.parseInt(numTickets);
                    
                    //create a ticket request object
                    TicketRequest2 tickReq =
                    new TicketRequest2(custNums[custIndex],
                    lastNames[custIndex], firstNames[custIndex],
                    cruiseIDs[cruiseIndex], cruiseDestinations[cruiseIndex],
                    cruisePorts[cruiseIndex], cruiseSailings[cruiseIndex],
                    numberOfTickets, false);
                    
                    //create the xml file
                    createXMLDoc(tickReq);
                    
                }
            }
            //See if you got a mail message back
            if (ae.getActionCommand().equals("Check"))
            {
                //Check to see if there are any email messages for us
                fetchMessages();
            }
            
        }catch (Exception e)
        {
            System.out.println("Exception thrown = " + e);
        }
    }
    
    private void fetchMessages()
    {
        try
        {
            Container c = btnExit.getParent();
            
            //read the text message from the file
            String dirName = "c:/XML/response/";
            File dir = new File(dirName);
            File f1;
            String messageLine = "";
            String strToken = "";
            String message = "";
            String message2 = "";
            
            String[]  fileList = dir.list();
            for (int i=0;i<fileList.length;++i)
            {
                String fileName = dirName + fileList[i];
                BufferedReader br = new BufferedReader(
                new FileReader(fileName));
                
                while( (messageLine = br.readLine()) != null)
                {
                    message += messageLine;
                    System.out.println("messageLine = " + messageLine);
                }
                StringTokenizer st = new StringTokenizer(message , "<>");
                while (st.hasMoreTokens())
                {
                    strToken = st.nextToken();
                    if (strToken.equals("ticketResponse"))
                    {
                        message2 = st.nextToken();
                        //show the message in a dialog box
                        JOptionPane.showMessageDialog(c, message2);
                    }
                }
                
            }
        }catch (Exception e)
        {
            System.out.println(" Exception " + e);
        }
    }
    
    
    public void createXMLDoc(TicketRequest2 tr2) throws IOException
    {
            try
            {
                
                DocumentBuilderFactory dFactory =
                DocumentBuilderFactory.newInstance();
                
                DocumentBuilder dBuilder = dFactory.newDocumentBuilder();
                
                doc = dBuilder.newDocument();
                
                //Create the root element
                Element ticketRequest = doc.createElement("ticketRequest");
                
                doc.appendChild(ticketRequest);
                
                //create the customer node
                Node customer = createCustomer(doc, tr2);
                ticketRequest.appendChild(customer);
                
                //create the cruise node
                Node cruise = createCruise(doc, tr2);
                ticketRequest.appendChild(cruise);
                
                
                //Create a transformer to write the file out
                TransformerFactory tFactory =
                TransformerFactory.newInstance();
                
                Transformer transformer = tFactory.newTransformer();
                DOMSource source = new DOMSource(ticketRequest);
                
                //Write out the file
                String filename = "c:/XML/request/request" +
                                    tr2.getCustID() + ".xml";
                StreamResult result = new StreamResult(
                new FileOutputStream(filename));
                transformer.transform(source, result);
                
                
            }catch(Exception e)
            {
                System.out.println("Exception e" + e);
            }
        }
        
        private Node createCustomer(Document doc, TicketRequest2 tr2)
        {
            Element lastName = doc.createElement("lastName");
            Element firstName = doc.createElement("firstName");
            
            lastName.appendChild(doc.createTextNode(tr2.getLastName()));
            firstName.appendChild(doc.createTextNode(tr2.getFirstName()));
            
            Element customer = doc.createElement("customer");
            
            Attr custIDAttribute = doc.createAttribute("custID");
            String s1 = String.valueOf(tr2.getCustID());
            custIDAttribute.setValue(s1);
            
            //append the attribute
            customer.setAttributeNode( custIDAttribute );
            customer.appendChild(lastName);
            customer.appendChild(firstName);
            
            return customer;
        }
        
        private Node createCruise(Document doc, TicketRequest2 tr2)
        {
            Element destination = doc.createElement("destination");
            Element port = doc.createElement("port");
            Element sailing = doc.createElement("sailing");
            Element numberOfTickets = doc.createElement("numberOfTickets");
            
            destination.appendChild(doc.createTextNode(tr2.getDestination()));
            port.appendChild(doc.createTextNode(tr2.getPort()));
            sailing.appendChild(doc.createTextNode(tr2.getSailing()));
            String s3 = String.valueOf(tr2.getNumberOfTickets());
            numberOfTickets.appendChild(doc.createTextNode(s3));
            
            Element cruise = doc.createElement("cruise");
            
            Attr cruiseIDAttribute = doc.createAttribute("cruiseID");
            String s2 = String.valueOf(tr2.getCustID());
            cruiseIDAttribute.setValue(s2);
            
            //append the attribute
            cruise.setAttributeNode(cruiseIDAttribute);
            cruise.appendChild(destination);
            cruise.appendChild(port);
            cruise.appendChild(sailing);       

            cruise.appendChild(numberOfTickets);
            
            return cruise;
        }
 
}