package unleashed.ch7;

import java.io.*;
import java.util.*;
import javax.transaction.*;
import javax.naming.*;
import javax.jms.*;
import java.sql.*;
import unleashed.TicketRequest2;

/**
 * The TicketAgent is a JMS and JDBC applicaiton.  It received a TicketRequest
 * object from the ticketRequestQueue.  It uses JDBC to access the database to
 * get the credit card number.  It then sends a message to the PaymentAgent
 * to request approval of a charge to the customer's credit card.  When the
 * charge is approved/disapproved TicketAgent updates the database and sends a message
 * to the CruiseList application.
 *
 *
 * @author Steve Potts
 */
public class TicketAgent implements MessageListener
{
    static int ticketID = 7000;
    private boolean quit = false;
    //JDBC variables
    javax.sql.DataSource ds = null;
    java.sql.Connection dbConn = null;
    Statement statement1 = null;
    
    //JMS variables
    QueueConnectionFactory qconFactory;
    QueueConnection qcon;
    QueueSession qsession;
    
    Queue ticketRequestQ;
    Queue ticketResponseQ;
    
    //create the senders
    QueueSender qTicketResponseSender;
    
    //create the receivers
    QueueReceiver qticketRequestReceiver;
    
    public TicketAgent() throws Exception
    {
        init();
    }
    
    //The init() method moves real processing out of the constructors where
    //Exception handling is simpler
    private void init() throws Exception
    {
        try
        {
            //Obtain connections to JDBC and JMS via JNDI
            ConnectToServices();
            
        }catch(Exception e)
        {
            System.out.println("Exception thrown " + e);
        }
    }
    
    private void ConnectToServices()
    {
        try
        {
            // Make connection to database
            // Obtain a Datasource connection from JNDI
            Context ctx = null;
            
            // Put connection properties in a hashtable.
            Hashtable ht = new Hashtable();
            ht.put(Context.INITIAL_CONTEXT_FACTORY,
            "weblogic.jndi.WLInitialContextFactory");
            ht.put(Context.PROVIDER_URL,
            "t3://localhost:7001");
            
            // Get a context for the JNDI look up
            ctx = new InitialContext(ht);
            
            //Get a connection to the datasource
            ds= (javax.sql.DataSource) ctx.lookup("examples-dataSource-demoPool");
            
            //Get a connection to the naming service
            qconFactory = (QueueConnectionFactory)
            ctx.lookup("weblogic.examples.jms.QueueConnectionFactory");
            qcon = qconFactory.createQueueConnection();
            qsession = qcon.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
            
            //get handles to all of the queues
            ticketRequestQ = (Queue)ctx.lookup("TicketRequestQueue2");
            ticketResponseQ = (Queue)ctx.lookup("TicketResponseQueue2");
            
            //create the senders
            qTicketResponseSender = qsession.createSender(ticketResponseQ);
            
            //create the receivers
            qticketRequestReceiver = qsession.createReceiver(ticketRequestQ);
            
            //Set the message Listeners
            qticketRequestReceiver.setMessageListener(this);
            
            qcon.start();
            System.out.println("JMS Connections created");
            
        } catch (Exception e)
        {
            System.out.println("Exception was thrown: " + e);
        }
    }
    
    /**
     * Message listener interface.
     * @param msg  message
     */
    
    // MessageListener interface method.  This method is run whenever a
    // queue that we are listening for receives a new message
    public void onMessage(Message msg)
    {
        try
        {
            String msgText = "";
            TicketRequest2 tr = null;
            Object o;
            ObjectMessage om;
            if (msg instanceof ObjectMessage)
            {
                //Dequeue the message
                om = (ObjectMessage)msg;
                o = om.getObject();
                //cast the message to the correct object type
                if (o instanceof TicketRequest2)
                {
                    tr = (TicketRequest2)o;
                    msgText = tr.toString();
                    
                    //get the credit card number from the database
                    String ccNum = getCreditCardNumber(tr.getCustID());
                    System.out.println("The Credit Card Number is " + ccNum);
                    
                    //create a queue send to the Payment queue
                    if (verifyCreditCard(ccNum))
                    {
                        createTicket(tr);
                        sendTicketResponse(tr, true);
                    }else
                        sendTicketResponse(tr, false);
                }
            }
            
            //if the queue receives a text message to quit, the process ends
            if (msgText.equalsIgnoreCase("quit"))
            {
                synchronized(this)
                {
                    quit = true;
                    this.notifyAll(); // Notify main thread to quit
                }
            }
        } catch (JMSException jmse)
        {
            jmse.printStackTrace();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    
    //This method sends a message to the response queue
    private void sendTicketResponse(TicketRequest2 tr, boolean isSuccessful)
    {
        try
        {
            //Create a message using this object
            String tickResp = "The ticket to " + tr.getDestination() + " for " +
            tr.getLastName();
            if (isSuccessful)
                tickResp += " was successful";
            else
                tickResp += " was not successful";
            
            System.out.println(tickResp);
            TextMessage tMessage = qsession.createTextMessage(tickResp);
            System.out.println("Created  the Ticket Response Message");
            
            //put the message on the queue
            qTicketResponseSender.send(tMessage);
            System.out.println("Sent the ticket Response Message");
        }catch(Exception e)
        {
            System.out.println("Exception " + e);
        }
    }
    
    //This method creates a database entry for the new ticket
    private void createTicket(TicketRequest2 tr) throws Exception
    {
        dbConn = ds.getConnection();
        System.out.println("createTicket Making Database connection...\n");
        
        //Create the statement
        statement1 = dbConn.createStatement();
        
        String insertStatement;
        ticketID += 1;
        insertStatement = "INSERT INTO CruiseTicket VALUES(" + ticketID + "," +
        Integer.toString(tr.getCustID()) + ", 'Unleashed Cruise Line'," +
        "'USS SeaBuscuit', '" + tr.getPort() + "','" + tr.getSailing() +"'," +
        "999.99," + "0,"+ "0,"+ "'')";
        
        statement1.executeUpdate(insertStatement);
        System.out.println("Update was successful CustID = " + tr.getCustID());
        
        statement1.close();
        dbConn.close();
        
    }
    
    private String getCreditCardNumber(int CustID) throws Exception
    {
        dbConn = ds.getConnection();
        System.out.println("getCreditCard Making Database connection...\n");
        
        //Create the statement
        statement1 = dbConn.createStatement();
        
        String ccNum = "";
        
        //Populate the creditcard string string
        String getCCString =
        "SELECT CreditCardNumber FROM CruiseCustomer " +
        "WHERE CustomerID = " + Integer.toString(CustID);
        
        ResultSet ccResults = statement1.executeQuery(getCCString);
        
        while (ccResults.next())
        {
            ccNum = ccResults.getString("CreditCardNumber");
        }
        
        statement1.close();
        dbConn.close();
        return ccNum;
    }
    
    private boolean verifyCreditCard(String ccNum)
    {
        int firstNum = Integer.parseInt(ccNum.substring(0,1));
        
        if ( firstNum > 4)
            return false;
        else
            return true;
    }
    
    /**
     * main() method.
     *
     * @exception  Exception if execution fails
     */
    public static void main(String[] args) throws Exception
    {
        try
        {
            TicketAgent ta = new TicketAgent();

            synchronized(ta)
            {
                while (!ta.quit)
                {
                    System.out.println("waiting on a message");
                    ta.wait();
                    System.out.println("received a message");
                }
            }
        }catch (Exception e)
        {
            System.out.println("Exception in main() " + e);
        }
    }
}