//Listing 10.5  The Java Transaction API (JTA) is capable of managing 
//transactions that span multiple databases.
/*
 * WlJTA.java
 *
 * Created on December 29, 2001, 11:39 AM
 */

package unleashed.ch10;

import java.sql.*;
import java.util.*;
import javax.naming.*;
import javax.transaction.*;

public class WlJTA
{
    
    public static void main(String argv[])
    throws Exception
    {
        java.sql.Connection dbConn = null;
        Statement statement1 = null;
        String createStatement;
        String insertStatement;
        
        try
        {
            // ============== Make connection to database =============
            // Obtain a Datasource from the WebLogic JNDI tree.
            Context ctx = null;
            
            // Put connection properties in to a hashtable.
            Hashtable ht = new Hashtable();
            ht.put(Context.INITIAL_CONTEXT_FACTORY,
            "weblogic.jndi.WLInitialContextFactory");
            ht.put(Context.PROVIDER_URL,
            "t3://localhost:7001");
            
            // Get a context for the JNDI look up
            ctx = new InitialContext(ht);
            javax.sql.DataSource ds
            = (javax.sql.DataSource) ctx.lookup(
                                 "examples-dataSource-demoPool");
            dbConn = ds.getConnection();
            System.out.println("Making connection...\n");
            
            //Get a handle to a UserTransaction object
            UserTransaction tx = 
                   (UserTransaction)ctx.lookup(
                            "javax.transaction.UserTransaction");
            System.out.println("Creating the transaction");
  
            
            //Create the statement
            statement1 = dbConn.createStatement();
            
            ////////////////////////////////////////////////////////////
            //   Create the tables in the database                    //
            ////////////////////////////////////////////////////////////
            try
            {
                statement1.execute("drop table CruiseReservation");
                statement1.execute("drop table CruiseCustomer");
                statement1.execute("drop table CruiseTicket");
                statement1.execute("drop table TravelAgency");
                statement1.execute("drop table CruiseBooking");
                System.out.println("all tables dropped.");
            } catch (SQLException e)
            {
                System.out.println("table doesn't need to be dropped.");
            }

            //Begin the transaction
            tx.begin();
            System.out.println("transaction has begun");
            
            //Add the customer data
            createStatement = 
              "CREATE TABLE CruiseReservation(resID int PRIMARY KEY,"
            + " CustomerID int, CruiseLine VARCHAR(30)," +
                " Ship VARCHAR(30), Port VARCHAR(30), "
            + "Sailing Date, AgencyID int, AgentName VARCHAR(30))";
            statement1.executeUpdate(createStatement);
            System.out.println("Table CruiseReservation created.");
            createStatement =
               "CREATE TABLE CruiseCustomer(CustomerID int PRIMARY KEY, "
            + "LastName VARCHAR(30), FirstName VARCHAR(30), " +
              "  MI VARCHAR(30), Address1 VARCHAR(30)," 
            + " Address2 VARCHAR(30), City VARCHAR(30), " +
             "  StateProv VARCHAR(30),"
            + " PostalCode VARCHAR(30), Country VARCHAR(30), " +
               "  CreditCardType VARCHAR(30),"
            + " CreditCardNumber VARCHAR(30), "
            + " isGambler BIT, isDrinker BIT, isShortTourist BIT, " +
              "  isPitchable BIT)";
            statement1.executeUpdate(createStatement);
            System.out.println("Table CruiseCustomer created.");
            createStatement = 
               "CREATE TABLE TravelAgency(AgencyID int PRIMARY KEY,"
            + " AgencyName VARCHAR(30), Address1 VARCHAR(30), " +
              " Address2 VARCHAR(30), City VARCHAR(30),"
            + " StateProv VARCHAR(30), PostalCode VARCHAR(30), " +
                 "  Country VARCHAR(30),"
            + " Telephone VARCHAR(30))";
            statement1.executeUpdate(createStatement);
            System.out.println("Table TravelAgency created.");
            createStatement = 
               "CREATE TABLE CruiseTicket(TicketID int PRIMARY KEY,"
            + " CustomerID int, CruiseLine VARCHAR(30), " +
              "  Ship VARCHAR(30), Port VARCHAR(30),"
            + " Sailing Date, Price FLOAT, AgencyID int, " +
             "  BookingID int, AgentName VARCHAR(30))";
            statement1.executeUpdate(createStatement);
            System.out.println("Table CruiseTicket created.");
            createStatement = 
               "CREATE TABLE CruiseBooking(BookingID int PRIMARY KEY,"
            + " TicketID int, CustomerID int, Smoking BIT, " +
               "  DiningPreference VARCHAR(30), "
            + "LangPref VARCHAR(30))";
            statement1.executeUpdate(createStatement);
            System.out.println("Table CruiseBooking created.");
            
            //Add the customer information
            insertStatement = "INSERT INTO CruiseCustomer VALUES(" +
                     " 1001, 'Carter', 'Joseph', 'D', "
            + "'123 4th St.', '', 'Kennesaw', 'GA', '10064', " +
                 "  'USA', 'MC', '1234-1234-1234-1234',"
            + "X'0',X'0',X'0',X'0')";
            statement1.addBatch(insertStatement);
          
            insertStatement = "INSERT INTO CruiseCustomer VALUES(" +
                " 1002, 'Cain', 'Clay', 'F',"
            + " '1203 5th St.', '', 'Macon', 'GA', '20064', " +
                  " 'USA', 'MC', '2234-2234-2234-2234', " 
            + " X'0',X'0',X'0',X'0')";
            statement1.addBatch(insertStatement);
            insertStatement = "INSERT INTO CruiseCustomer VALUES(" +
                  " 1003, 'Bush', 'Albert', 'R',"
            + " '1523 6th St.', '', 'Cincinnati', 'OH', '30064', " +
             "  'USA', 'MC', '3234-3234-3234-3234', "
            + " X'0',X'0',X'0',X'0')";
            statement1.addBatch(insertStatement);
            insertStatement = "INSERT INTO CruiseCustomer VALUES(" +
                   " 1004, 'Skelton', 'Allison', 'T',"
            + " '1523 7th St.', '', 'Middletown', 'AL', '40064', " +
              " 'USA', 'MC', '4234-4234-4234-4234', "
            + "X'0',X'0',X'0',X'0')";
            statement1.addBatch(insertStatement);
            insertStatement = "INSERT INTO CruiseCustomer VALUES(" +
                  " 1005, 'Davidson', 'Barco', 'G'," 
            + " '1823 8th St.', '', 'Dayton', 'OH', '50064', " + 
               "  'USA', 'MC', '5234-5234-5234-5234', "
            + " X'0',X'0',X'0',X'0')";
            statement1.addBatch(insertStatement);
            insertStatement = "INSERT INTO CruiseCustomer VALUES(" +
               " 1006, 'Rangitch', 'Maureen', 'B',"
            + " '1223 9th St.', '', 'Cairo', 'KA', '60064', " +
               "  'USA', 'MC', '6234-6234-6234-6234', " 
            + " X'0',X'0',X'0',X'0')";
            statement1.addBatch(insertStatement);
            insertStatement = "INSERT INTO CruiseCustomer VALUES(" +
                            " 1007, 'Milner', 'Grant', 'V', " 
            + "'1293 14th St.', '', 'Auberna', 'TN', '70064', " +
                   "  'USA', 'MC', '7234-7234-7234-7234', "
            + " X'0',X'0',X'0',X'0')";
            statement1.addBatch(insertStatement);
            insertStatement = "INSERT INTO CruiseCustomer VALUES(" +
                         " 1008, 'Walker', 'Mike', 'C'," 
            + " '1223 24th St.', '', 'Spokane', 'WA', '80064', " +
                  "  'USA', 'MC', '8234-8234-8234-8234', "
            + " X'0',X'0',X'0',X'0')";
            statement1.addBatch(insertStatement);
            insertStatement = "INSERT INTO CruiseCustomer VALUES(" +
                   " 1009, 'Cheney', 'Michael', 'L',"
            + " '1723 25th St.', '', 'Orlando', 'FL', '90064', " +
               "  'USA', 'MC', '9234-9234-9234-9234', "
            + " X'0',X'0',X'0',X'0')";
            statement1.addBatch(insertStatement);
            insertStatement = "INSERT INTO CruiseCustomer VALUES(" +
             " 1010, 'Pfister', 'Darlene', 'D'," 
            + " '1233 9th St.', '', 'Bangor', 'MA', '10064', " +
                         "  'USA', 'MC', '0234-0234-0234-0234', "
            + " X'0',X'0',X'0',X'0')";
            statement1.addBatch(insertStatement);
             
            //Execute the batch
            statement1.executeBatch();
             
            //add the Travel Agents
            insertStatement = 
            "INSERT INTO TravelAgency VALUES(" +
                    " 2001, 'Cruise Junction', '1233 Sego St.', " 
            + "'', 'Miami', 'FL', '10064', 'USA', '(800)242-4242')";
            statement1.addBatch(insertStatement);
            insertStatement =
            "INSERT INTO TravelAgency VALUES(" +
                    " 2002, 'Cruise and Snooze', '9387 Atlantic Ave.', " 
            + "'', 'New York', 'NY', '00064', 'USA', '(800)343-3434')";
            statement1.addBatch(insertStatement);
            insertStatement =
            "INSERT INTO TravelAgency VALUES(" +
                       " 2003, 'Pokeys Cruises', '98223 Baltic Dr.', "
            + "'', 'Los Angeles', 'CA', '20064', 'USA', '(800)565-6565')";
            statement1.addBatch(insertStatement);
            insertStatement = 
            "INSERT INTO TravelAgency VALUES(" +
                " 2004, 'Cruise Animal', '2345 Lester Rd.', " 
            + "'', 'New Orleans', 'LA', '40064', 'USA', '(800)787-8787')";
            statement1.addBatch(insertStatement);
            insertStatement = 
            "INSERT INTO TravelAgency VALUES(" +
                " 2005, 'Cruisy Dot Com', '5435 Spring St.', " 
            + " '', 'Atlanta', 'GA', '30064', 'USA', '(800)879-6543')";
            statement1.addBatch(insertStatement);
            
            //Execute the batch
            statement1.executeBatch();
            
            //commit the transaction
            tx.commit();
            System.out.println(
                        "All rows added successfully and committed");

        } catch (Exception e)
        {
            System.out.println("Exception was thrown: " +
                                                  e.getMessage());
        } finally
        {
            try
            {
                if (statement1 != null)
                    statement1.close();
                if (dbConn != null)
                    dbConn.close();
            } catch (SQLException sqle)
            {
                System.out.println("SQLException during close(): " +
                                                      sqle.getMessage());
            }
        }
    }
    
}