/*
 * SimplePopClientEvent.java
 *
 * Created on April 26, 2002, 2:37 PM
 */

package unleashed.ch13;

import javax.activation.*;
import javax.mail.*;
import javax.mail.internet.*;
import javax.mail.event.*;

import java.util.*;
import java.io.*;

/**
 *
 * @author  Stephen Potts, Craig Pfeifer
 * @version
 */
public class SimplePopClientEvent
{
    String server;
    String user;
    String password;
    static Folder folder1 = null;
    static Store store1 = null;
    Session session;

    /** Creates new SimplePopClient */
    public SimplePopClientEvent()
    {
        setup();
    }

    private void setup()
    {
        try
        {
            //change these values to match your own personal settings
            server = "<SERVERNAME>";
            user = "UID";
            password = "PWD";
            Properties props = System.getProperties();
            props.put("mail.imap.host", "<YOUR IMAP HOST HERE>");
            props.put("mail.smtp.host", "<YOUR SMTP HOST HERE>");
            session = Session.getDefaultInstance(props, null);

            //get a handle to the  store
            store1 = session.getStore("imap");

            //connect to the store and pass the authentication
            store1.connect(server, user, password);

            //obtain the default folder handle
            folder1 = store1.getDefaultFolder().getFolder("Inbox");

            //if there is no inbox, throw an exception
            if (folder1 != null)
            {
                folder1.open(Folder.READ_ONLY);
            }else
            {
                System.out.println("No such folder!!");
            }
            folder1.addMessageCountListener(
                         new MyMessageCountListener());

        }
        catch (Exception e)
        {
            log("Exception thrown in setup() " + e);
            e.printStackTrace();
        }
        System.out.println(
                 "Got connection to mailserver & account INBOX");
    }

    private void fetchMessages()
    {
        try
        {

        //if there are any messages, retrieve them
        Message[] messages = folder1.getMessages();
        if (messages.length == 0)
        {
            log("There are no messages to retrieve");
        }

        //print out the messages
        for (int i=0; i<messages.length; i++)
        {
            logMessage(messages[i]);
        }
        }catch (Exception e)
        {
            System.out.println("error while fetching");
        }
    }

    public void logMessage(Message message1)
    {
        try
        {
            log("----------------------------------------");
            //get the From address
            InternetAddress ia = (InternetAddress)message1.getFrom()[0];

            //get the personal name from the message.  This is the name 
            //in human readabel form like Jake Snyder.
            String personalName = ia.getPersonal();
            log("The personal name is " + personalName);

            //get the addressString like jakesnyder@earthlink.com
            String addressString = ia.getAddress();
            log("The address String is " + addressString);

            //get the contents of the subject line
            String subject = message1.getSubject();
            log("The subject is " + subject);

            //Get the body of the message
            Part messagePart1 = message1;
            Object content = messagePart1.getContent();
            String contentType = messagePart1.getContentType();

            log("The content type is " + contentType);

            //If the message is plain text then you can read it with
            //a StreamReader
            if (contentType.startsWith("text/plain") ||
            contentType.startsWith("text/html"))
            {
                InputStream is = messagePart1.getInputStream();

                BufferedReader br = new BufferedReader(
                                    new InputStreamReader(is));
                String line1=br.readLine();

                while (line1 != null)
                {
                    log(line1);
                    line1=br.readLine();
                }
            }
            log("----------------------------------------");
        }
        catch (Exception e)
        {
            log("Exception in logMessage " + e);
        }
    }

    private void log(String s)
    {
        System.out.println(s);
    }

    public static void main(String[] args)
    {
        //Instantiate the object
        SimplePopClientEvent spce = new SimplePopClientEvent();
        //get the messages
        while (true)
        {
            try
            {

                Thread.sleep(1000);
                System.out.println("woke up");
                int x  = folder1.getMessageCount();
                System.out.println("message count = "+x);

                boolean newMsgs = folder1.hasNewMessages();
                System.out.println("has new messags = "+newMsgs);
            }catch (InterruptedException ie)
            {

                try
                {
                    if (folder1 != null) folder1.close(false);
                    if (store1 != null) store1.close();
                }
                catch (Exception ee)
                { System.out.println("Problem closing " + ee); }
            }
            catch (MessagingException me)
            {
              System.out.println("Problem getting message count " + me);
            }
        }
    }
}