/*
 * SimplePopClientSearch.java
 *
 * Created on April 25, 2002, 2:37 PM
 */

package unleashed.ch13;

import javax.mail.*;
import javax.mail.search.*;
import javax.mail.internet.*;

import java.util.*;
import java.io.*;

/**
 *
 * @author  Stephen Potts
 * @version
 */
public class SimplePopClientSearch
{
    String server;
    String user;
    String password;
    
    /** Creates new SimplePopClient */
    public SimplePopClientSearch()
    {
        setup();
    }
    
    private void setup()
    {
        //change these values to match your own personal settings
        server = "mail.bellsouth.net";
        user = "sapotts";
        password = "xyzxyz";
    }
    
    private void fetchMessages()
    {
        Store store1 = null;
        Folder folder1 = null;
        try
        {        
            Properties props = System.getProperties();
            Session session = Session.getDefaultInstance(props, null);
            
            //get a handle to the pop3 store
            store1 = session.getStore("pop3");
            
            //connect to the store and pass the authentication
            store1.connect(server, user, password);
            
            //obtain the default folder handle
            folder1 = store1.getDefaultFolder();
            
            //get the handle to the inbox
            folder1 = folder1.getFolder("INBOX");
            
            //if there is no inbox, throw an exception
            if (folder1 != null)
            {
                folder1.open(Folder.READ_ONLY);
            }else
                throw new Exception("no inbox available");
            
             //if there are any messages search for them
            SearchTerm criteria = new OrTerm( new SubjectTerm("Ch13"),
                                  new SubjectTerm("Chapter 13"));
            
            Message[] messages = folder1.search(criteria);
            if (messages.length == 0)
            {
                log("There are no messages to retrieve");
            }
            
            //print out the messages
            for (int i=0; i<messages.length; i++)
            {
                logMessage(messages[i]);
            }
            
        }catch (Exception e)
        {
            log("Exception thrown in fetchMessages() " + e);
        }
        finally
        {
            try
            {
                if (folder1 != null) folder1.close(false);
                if (store1 != null) store1.close();
            }
            catch (Exception ee)
            { log("Problem closing " + ee); }
        }
    }
    
    public void logMessage(Message message1)
    {
        try
        {
            log("----------------------------------------");
            //get the From address
            InternetAddress ia = (InternetAddress)message1.getFrom()[0];
            
            //get the personal name from the message.  This is the name in 
            //human readabel form like Jake Snyder.
            String personalName = ia.getPersonal();
            log("The personal name is " + personalName);
            
            //get the addressString like jakesnyder@earthlink.com
            String addressString = ia.getAddress();
            log("The address String is " + addressString);
            
            //get the contents of the subject line
            String subject = message1.getSubject();
            log("The subject is " + subject);
            
            //Get the body of the message
            Part messagePart1 = message1;
            Object content = messagePart1.getContent();
            String contentType = messagePart1.getContentType();
            
            log("The content type is " + contentType);
            
            //If the message is plain text then you can read it with
            //a StreamReader
            if (contentType.startsWith("text/plain") ||
            contentType.startsWith("text/html"))
            {
                InputStream is = messagePart1.getInputStream();
                
                BufferedReader br = new BufferedReader(new InputStreamReader(is));
                String line1=br.readLine();
                
                while (line1 != null)
                {
                    log(line1);
                    line1=br.readLine();
                }
            }
            log("----------------------------------------");
        }
        catch (Exception e)
        {
            log("Exception in logMessage " + e);
        }
    }
    
    private void log(String s)
    {
        System.out.println(s);
    }
    
    public static void main(String[] args)
    {
        //Instantiate the object
        SimplePopClientSearch spcs = new SimplePopClientSearch();
        //get the messages
        spcs.fetchMessages();
    }
}

