//Listing 13.4 The TicketAgent is responsible for receiving the message, parsing it, //checking the credit card, updating the database, and sending a response to the //CruiseList GUI. 
/*
 * TicketAgent.java
 *
 * Created on January 14, 2002, 5:23 PM
 */

package unleashed.ch13;

/**
 *
 * @author  Stephen Potts
 * @version
 */

import java.io.*;
import java.util.*;
import javax.transaction.*;
import javax.naming.*;

import java.sql.*;
import javax.mail.*;
import javax.mail.internet.*;

/**
 * The TicketAgent is a JavaMail and JDBC applicaiton.  It receives a Ticket Request
 * object from the CruiseList GUI.  It uses JDBC to access the database to
 * get the credit card number.  It then sends a message to the PaymentAgent
 * to request approval of a charge to the customer's credit card.  When the
 * charge is approved/disapproved TicketAgent updates the database and sends a message
 * back to the CruiseList application.
 *
 *
 * @author Steve Potts
 */
public class TicketAgent
{
    static int ticketID = 7000;
    private boolean quit = false;
    //JDBC variables
    java.sql.Connection dbConn = null;
    Statement statement1 = null;
    
    //JavaMail variables
    String server;
    String user;
    String password;
    
    String smtpServer;
    String to;
    String from;
    String subject;
    String body;
    
    
    public TicketAgent() throws Exception
    {
        init();
    }
    
    //The init() method moves real processing out of the constructors where
    //Exception handling is simpler
    private void init() throws Exception
    {
        try
        {
            //Obtain connections to JDBC and JavaMail
            ConnectToServices();
            
        }catch(Exception e)
        {
            System.out.println("Exception thrown " + e);
        }
    }
    
    private void ConnectToServices()
    {
        try
        {
            //Get a connection to the datasource
             //load the driver class
            Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");       
           
            //Specify the ODBC data source
            String sourceURL = "jdbc:odbc:CruiseTicket";
            
            //get a connection to the database
            dbConn =DriverManager.getConnection(sourceURL);
            
            //If we get to here, no exception was thrown
            System.out.println("The database connection is " + dbConn);
            System.out.println("Making connection...\n");
            
            //Create the statement
            statement1 = dbConn.createStatement();
           
           
          
            //Get a connection to the JavaMail POP services
            server = "mail.bellsouth.net";
            user = "sapotts";
            password = "xyzxyz";
            
            //Get a connection to the SMTP service
            //This is where you put in the settings for your message
            smtpServer = "mail.bellsouth.net";
            to = "sapotts@bellsouth.net";
            from = "sapotts@bellsouth.net";
            subject = "Response";
            //body = "Mindspring.com This is a test to see if respond to a message";
            
            
            System.out.println("JavaMail Connections created");
            
        } catch (Exception e)
        {
            System.out.println("Exception was thrown: " + e);
        }
    }
    
    private void sendResponse(String s)
    {
        try
        {
            body = s;
            log("Sending a message to " + to);
            
            //This is the code that does the sending of the message
            Properties props = System.getProperties();
            
            props.put("mail.smtp.host", smtpServer);
            Session session = Session.getDefaultInstance(props, null);
            
            //Create the new message
            MimeMessage msg = new MimeMessage(session);
            
            //Set the send from address
            msg.setFrom(new InternetAddress(from));
            
            //Set the to address
            msg.setRecipients(Message.RecipientType.TO,
            InternetAddress.parse(to, false));
            
            //Set the subject
            msg.setSubject(subject);
            msg.setText(body);
            
            //set the date
            msg.setSentDate(new java.util.Date());
            
            //send the message
            Transport.send(msg);
            
            System.out.println("Message was sent");
        }catch (Exception e)
        {
            log("Error sending message  " + e);
        }
    }
    
    public void respondToRequest(TicketRequest tr)
    {
        try
        {
            String msgText = "";
            
            //get the credit card number from the database
            String ccNum = getCreditCardNumber(tr.getCustID());
            System.out.println("The Credit Card Number is " + ccNum);
            
            //create a queue send to the Payment queue
            if (verifyCreditCard(ccNum))
            {
                createTicket(tr);
                sendTicketResponse(tr, true);
            }else
                sendTicketResponse(tr, false);
            
            //if the queue receives a text message to quit, the process ends
            if (msgText.equalsIgnoreCase("quit"))
            {
                synchronized(this)
                {
                    quit = true;
                    this.notifyAll(); // Notify main thread to quit
                }
            }
        }catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    
    //This method sends a message to the response queue
    private void sendTicketResponse(TicketRequest tr, boolean isSuccessful)
    {
        try
        {
            //Create a message using this object
            String tickResp = "The ticket to " + tr.getDestination() + " for " +
            tr.getLastName();
            if (isSuccessful)
                tickResp += " was successful";
            else
                tickResp += " was not successful";
            
            System.out.println(tickResp);
            System.out.println("Created  the Ticket Response Message");
            
            //send the response
            sendResponse(tickResp);
            System.out.println("Sent the ticket Response Message");
        }catch(Exception e)
        {
            System.out.println("Exception " + e);
        }
    }
    
    private TicketRequest createTicketRequestFromEmail(String s1)
    {
        log("createTicketRequestFromEmail() " + s1);
        StringTokenizer st = new StringTokenizer(s1, ",");
        String custString = st.nextToken();
        int custID = Integer.parseInt(custString);
        String lastName = st.nextToken();
        String firstName = st.nextToken();
        String cruiseIDString = st.nextToken();
        int cruiseID = Integer.parseInt(cruiseIDString);
        String destination = st.nextToken();
        String port = st.nextToken();
        String sailing = st.nextToken();
        String numTicketString = st.nextToken();
        int numTickets = Integer.parseInt(numTicketString);
        
        TicketRequest ticketReq = new TicketRequest(custID,
        lastName, firstName,
        cruiseID, destination,
        port, sailing,
        numTickets);
        log("ticketReq = " + ticketReq);
        return ticketReq;
    }
    
    //This method creates a database entry for the new ticket
    private void createTicket(TicketRequest tr) throws Exception
    {
        
        String insertStatement;
        ticketID += 1;
        insertStatement = "INSERT INTO CruiseTicket VALUES(" + ticketID + "," +
        Integer.toString(tr.getCustID()) + ", 'Unleashed Cruise Line'," +
        "'USS SeaBuscuit', '" + tr.getPort() + "','" + tr.getSailing() +"'," +
        "999.99," + "0,"+ "0,"+ "'')";
        
        statement1.executeUpdate(insertStatement);
        System.out.println("Update was successful CustID = " + tr.getCustID());
    }
    
    private String getCreditCardNumber(int CustID) throws Exception
    {
        System.out.println("getCreditCard Making Database connection...\n");
        
        String ccNum = "";
        
        //Populate the creditcard string string
        String getCCString =
        "SELECT CreditCardNumber FROM CruiseCustomer " +
        "WHERE CustomerID = " + Integer.toString(CustID);
        
        ResultSet ccResults = statement1.executeQuery(getCCString);
        
        while (ccResults.next())
        {
            ccNum = ccResults.getString("CreditCardNumber");
        }
        
        return ccNum;
    }
    
    private boolean verifyCreditCard(String ccNum)
    {
        int firstNum = Integer.parseInt(ccNum.substring(0,1));
        
        if ( firstNum > 4)
            return false;
        else
            return true;
    }
    
    private void fetchMessages()
    {
        Store store1 = null;
        Folder folder1 = null;
        try
        {
            Properties props = System.getProperties();
            Session session = Session.getDefaultInstance(props, null);
            
            //get a handle to the pop3 store
            store1 = session.getStore("pop3");
            
            //connect to the store and pass the authentication
            store1.connect(server, user, password);
            
            //obtain the default folder handle
            folder1 = store1.getDefaultFolder();
            
            //get the handle to the inbox
            folder1 = folder1.getFolder("INBOX");
            
            //if there is no inbox, throw an exception
            if (folder1 != null)
            {
                folder1.open(Folder.READ_WRITE);
            }else
                throw new Exception("no inbox available");
            
            //if there are any messages, retrieve them
            Message[] messages = folder1.getMessages();
            if (messages.length == 0)
            {
                log("There are no messages to retrieve");
            }
            
            //print out the messages
            for (int i=0; i<messages.length; i++)
            {
                logMessage(messages[i]);
            }
            
        }catch (Exception e)
        {
            log("Exception thrown in fetchMessages() " + e);
        }
        finally
        {
            try
            {
                if (folder1 != null) folder1.close(true);
                if (store1 != null) store1.close();
                log("finalize worked fine");
            }
            catch (Exception ee)
            {
                log("Problem closing " + ee);
            }
        }
    }
    
    public void logMessage(Message message1)
    {
        try
        {
            log("----------------------------------------");
            //get the From address
            InternetAddress ia = (InternetAddress)message1.getFrom()[0];
            
            //get the personal name from the message.  This is the name in
            //human readabel form like Jake Snyder.
            String personalName = ia.getPersonal();
            log("The personal name is " + personalName);
            
            //get the addressString like jakesnyder@earthlink.com
            String addressString = ia.getAddress();
            log("The address String is " + addressString);
            
            //get the contents of the subject line
            String subject = message1.getSubject();
            log("The subject is " + subject);
            if (subject.equals("Request"))
            {
                //Get the body of the message
                Part messagePart1 = message1;
                Object content = messagePart1.getContent();
                String contentType = messagePart1.getContentType();
                
                log("The content type is " + contentType);
                
                //If the message is plain text then you can read it with
                //a StreamReader
                if (contentType.startsWith("text/plain") ||
                contentType.startsWith("text/html"))
                {
                    InputStream is = messagePart1.getInputStream();
                    
                    BufferedReader br = new BufferedReader(new InputStreamReader(is));
                    String line1=br.readLine();
                    
                    while (line1 != null)
                    {
                        log(line1);
                        //evaluate the request
                        TicketRequest tr = createTicketRequestFromEmail(line1);
                        respondToRequest(tr);
                        line1=br.readLine();
                    }
                }
                message1.setFlag(Flags.Flag.DELETED, true);
            }
            log("----------------------------------------");
        }
        catch (Exception e)
        {
            log("Exception in logMessage " + e);
        }
    }
    
    private void log(String s)
    {
        System.out.println(s);
    }
    
    /**
     * main() method.
     *
     * @exception  Exception if execution fails
     */
    public static void main(String[] args) throws Exception
    {
        try
        {
            TicketAgent ta = new TicketAgent();
            
            synchronized(ta)
            {
                while (!ta.quit)
                {
                    System.out.println("Starting the wait");
                    ta.wait(30000);
                    ta.fetchMessages();
                    System.out.println("Just woke up");
                }
            }
        }catch (Exception e)
        {
            System.out.println("Exception in main() " + e);
        }
    }
}
