/*
 * PortReader.java
 *
 * Created on February 8, 2002, 4:40 PM
 */

package unleashed.ch14;
import java.io.*;
import javax.comm.*;
import java.util.*;

/**
 *
 * @author  Stephen Potts
 * @version
 */
public class PortReader implements Runnable, SerialPortEventListener
{
    static Enumeration ports;
    static CommPortIdentifier pID;
    InputStream inStream;
    SerialPort serPort;
    Thread readThread;
    
    /** Creates new PortFinder */
    public PortReader()
    {
        try
        {
            serPort = (SerialPort)pID.open("PortReader",2000);
        }catch (PortInUseException piue)
        {
            System.out.println("Exception " + piue);
        }

        try
        {
            inStream = serPort.getInputStream();
        }catch (IOException ioe)
        {
            System.out.println("Exception " + ioe);
        }
         
        try
        {
            serPort.addEventListener(this);
        }catch (TooManyListenersException tmle)
        {
            System.out.println("Exception " + tmle);
        }
        
        serPort.notifyOnDataAvailable(true);
        
        try
        {
            serPort.setSerialPortParams(9600, SerialPort.DATABITS_8,
            SerialPort.STOPBITS_1,
            SerialPort.PARITY_NONE);
        }catch (UnsupportedCommOperationException ucoe)
        {
            System.out.println("Exception " + ucoe);
        }
        readThread = new Thread(this);
        readThread.start();
    }//Constructor
    
    public void serialEvent(SerialPortEvent event)
    {
        switch(event.getEventType())
        {
            case SerialPortEvent.BI:
                System.out.println("SerialPortEvent.BI occurred");
            case SerialPortEvent.OE:
                System.out.println("SerialPortEvent.OE occurred");
            case SerialPortEvent.FE:
                System.out.println("SerialPortEvent.FE occurred");
            case SerialPortEvent.PE:
                System.out.println("SerialPortEvent.PE occurred");
            case SerialPortEvent.CD:
                System.out.println("SerialPortEvent.CD occurred");
            case SerialPortEvent.CTS:
                System.out.println("SerialPortEvent.CTS occurred");
            case SerialPortEvent.DSR:
                System.out.println("SerialPortEvent.DSR occurred");
            case SerialPortEvent.RI:
                System.out.println("SerialPortEvent.RI occurred");
            case SerialPortEvent.OUTPUT_BUFFER_EMPTY:
                System.out.println("SerialPortEvent.OUTPUT_BUFFER_EMPTY occurred");
                break;
            case SerialPortEvent.DATA_AVAILABLE:
                System.out.println("SerialPortEvent.DATA_AVAILABLE occurred");
                byte[] readBuffer = new byte[20];
                
                try
                {
                    while (inStream.available() > 0)
                    {
                        int numBytes = inStream.read(readBuffer);
                    }
                    System.out.print(new String(readBuffer));
                 }catch (IOException ioe)
                {
                    System.out.println("Exception " + ioe);
                }
                break;
        }//switch
    }//serialEvent
    
    public static void main(String[] args)
    {
        ports = CommPortIdentifier.getPortIdentifiers();
        
        while(ports.hasMoreElements())
        {
            pID = (CommPortIdentifier)ports.nextElement();
            System.out.println("Port " + pID.getName());
            
            if (pID.getPortType() == CommPortIdentifier.PORT_SERIAL)
            {
                if (pID.getName().equals("COM1"))
                {
                    PortReader pReader = new PortReader();
                    System.out.println("COM1 found");
                }
            }
        }
    }//main()
    
    public void run()
    {
        try
        {
            Thread.sleep(20000);
        }catch (InterruptedException ie)
        {
            System.out.println("Exception " + ie);
        }
    }//run()
}
