/*
 * MinimalVideoPlayer.java
 *
 * Created on February 12, 2002, 10:49 AM
 */

package unleashed.ch17;

import javax.media.*;
import javax.media.format.*;
import javax.media.protocol.*;

import java.awt.*;
import java.awt.event.*;

/**
 *
 * @author  Stephen Potts
 * @version
 */
public class MinimalVideoPlayer extends Frame
                  implements ControllerListener, ActionListener
{
    Player videoPlayer;
    
    String filename;
    
   
    /** Creates new MinimalVideoPlayer */
    public MinimalVideoPlayer(String s1)
    {
        super(s1);
        
        //handle window event handler
        addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent we)
            {
                dispose();
                System.exit(0);
            }
        }); 
        
        //create the menu
        MenuBar mb = new MenuBar();
        setMenuBar(mb);
        Menu fileMenu = new Menu("File");
        mb.add(fileMenu);
        
        MenuItem itemPlay = new MenuItem("Play");
        itemPlay.addActionListener(this);
        fileMenu.add(itemPlay);
    }
 
    //add the event handler
    public void actionPerformed(ActionEvent ae)
    {
        String action = ae.getActionCommand().toString();
        if (action.equals("Play"))
        {
            play();
        }
    }
    
    //play the video  
    void play()
    {
        try
        {
            //ask the user for a file name
            FileDialog fd = 
               new FileDialog(this, "Choose Video", FileDialog.LOAD);
            fd.show();
            
            //assemble the filename
            filename = fd.getDirectory() + fd.getFile();
            
            //create the player
            videoPlayer = Manager.createPlayer(
                new MediaLocator("file:///" + filename));
            
            //add a listener for player state changes
            System.out.println("Adding controller listener");
            videoPlayer.addControllerListener(this);
            
            videoPlayer.start();
        }catch (Exception e)
        {
            System.out.println("Error " + e);
        }
    }//play
           
    public synchronized void controllerUpdate(
                            ControllerEvent event)
    {
        System.out.println("Event: " + event);
        
        if (event instanceof RealizeCompleteEvent)
        {
            Component comp;
            
            if ((comp = videoPlayer.getVisualComponent()) != null)
            {
                add("Center", comp);
            }else
                System.out.println("Unable to get visual component");
            
            validate();
        }
    }
    
    public static void main(String[] args)
    {
        MinimalVideoPlayer mvp = new MinimalVideoPlayer("MinimalVideoPlayer");
        
        //show the frame containing the video
        mvp.show();
        //size the window
        mvp.setSize(300,300);
        mvp.setLocation(250,300);
    }    
}