package unleashed.ch20.tempmethod;

/**
* Template method pattern example.
*/
public class Application
{
    protected Document doc;
    
    public Application()
    {
        System.out.println("Application is created");
    }
    
    public static void main(String[] args)
    {
        Application app = new Application();
        
        app.openDocument();
    }
    
    //Defines algorighm to open a document.
    public void openDocument()
    {
        doc = new ConcreteDocument();
        if (doc.exists())
        {
            doc.beforeOpen();
            doc.open();
            doc.afterOpen();
        }
    }
    
    
}

/**
* Declares required operations for a document
*/
interface Document
{
    public boolean exists();
    public void beforeOpen();
    public String open();
    public void afterOpen();
}

/**
* Provides a base implementation for a document, 
* specifying primitive operations and hooks.
*/
abstract class BaseDocument implements Document
{

    public abstract boolean exists();
    
    public void beforeOpen()
    {
        System.out.println("BaseDocument: default hook 'beforeOpen' is called");
    }
    
    public abstract String open();
    
    public void afterOpen()
    {
        System.out.println("BaseDocument: default hook 'afterOpen' is called");
    }
}

/**
* Implements a concrete document. Must implement the 
* primitive operaitions only. May optionally implement the hooks. 
*/
class ConcreteDocument extends BaseDocument
{
    //primitive operation exists implementation
    public boolean exists()
    {
        System.out.println("ConcreteDocument: primitive operation 'exists' is called");
           
        //lets say that this document exists already
        return true;
    }
    
    //primitive operaiton open implementation
    public String open()
    {
        System.out.println("ConcreteDocument: primitive operation 'open' is called");
        return "New Document";
    }
    
    //examle of a hook implementaiton
    public void afterOpen()
    {
        System.out.println("ConcreteDocument: hook 'afterOpen' is called");
    }
}


