/**
* Proxy design pattern example
*/

package unleashed.ch21.proxy;

/**
* Property file interface
*/
interface PropertyFile
{

	public String getFileName();
	
	public String getPropertyValue(String propertyName);
}

/**
* Property file real object
*/
class PropertyFileImpl implements PropertyFile
{
	protected String fileName;

	/**
	* Loads a property file
	*/
	public PropertyFileImpl(String fileName)
	{
		this.fileName = fileName;
		//load property a large file from disk 
		System.out.println("Creating real file and loading into memory");
	}

	/**
	* Gets file name.
	*/
	public String getFileName()
	{
		return fileName;
	}	

	/**
	* Gets the value of the property
	*/
	public String getPropertyValue(String propertyName)
	{
		//get property value from the file
		return "Reading property value from file";
	}
}

/**
* Property file proxy, mimics property file interface
*/
class PropertyFileProxy implements PropertyFile
{
	public PropertyFileProxy()
	{
		System.out.println("Creating file proxy");
	}
	//properties file name
	protected String fileName = "properties.in";

	//real object 
	protected PropertyFile realObject;
	/**
	* Gets the name of the properties file without loading it.
	*/
	public String getFileName()
	{
		return fileName;
	}

	/**
	* Gets the value of a property. 
	*/
	public String getPropertyValue(String propertyName)
	{
		
		if(realObject == null)
		{
			//we are forced to load the file at this  point
			realObject = new PropertyFileImpl(fileName);
		}
				
		return realObject.getPropertyValue(propertyName);
			
	}
}

public class ProxyExample
{
	public static void main(String[] args)
	{
		ProxyExample example = new ProxyExample();
		
		PropertyFileProxy propertyFile = new PropertyFileProxy();
		
		example.usePropertyFile(propertyFile);
	}
	
	public void usePropertyFile(PropertyFile properties)
	{
	
		System.out.println("Property file name " + properties.getFileName());
		
		System.out.println(properties.getPropertyValue("myproperty") );
		
	}
}
