package unleashed.ch23;

import java.io.*;
import java.net.*;
import java.security.*;

import javax.net.*;
import javax.net.ssl.*;

/**
* SSL Server class runs in a separate thread. Reads the client data 
* then sends response back to the client
*/
class MySSLServer extends Thread
{
	private Socket socket;
	
	public MySSLServer(Socket socket)
	{
		this.socket = socket;
	}
	
	public void run()
	{
		try
		{
			System.out.println("Connection established");
            BufferedReader r = new BufferedReader(new InputStreamReader(socket.getInputStream()));
			
			PrintWriter p = new PrintWriter(socket.getOutputStream());
			
            System.out.println("Reading data");
			String data = r.readLine();
			System.out.println("Data received: " + data);
			
            System.out.println("Sending response");
            
			p.println("HELLO FROM SSL SERVER");
			
			p.close();
            
			System.out.println("Data received..closing connection");
			socket.close();
            
		}catch(IOException e)
		{
		    System.out.println("IOException is server: " + e.getMessage());
		}
	}

}

/**
* Server main class
*/
public class SSLServerExample extends Thread
{
    /**
    * Creates SSL socket and listens on it.
    */
	public static void main(String[] args)
	{
		try
		{
            //creates SSL socket factory
			ServerSocketFactory sf = SSLServerSocketFactory.getDefault();
           
            //creates SSL socket
			SSLServerSocket ss = (SSLServerSocket)sf.createServerSocket(9096);
			
            //enables truststore cipher suite
			enableAllCipherSuites(ss);
			
            		
		    System.out.println("Server started...listening");	
			while (true)
			{
                //listen in a while loop
                new MySSLServer(ss.accept()).start();
			}
		}catch(Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
    /**
    * Enables the cipher suite that is used in the truststore.
    */
	private static void enableAllCipherSuites(SSLServerSocket ss)
	{
			String newsuites[] = new String[8];
			
            //read all enabled cipher suites
			String suites[] = ss.getEnabledCipherSuites();
            
            //overwrite the first cell with our cipher suite
			suites[0] = "SSL_DH_anon_EXPORT_WITH_DES40_CBC_SHA";
		    
            //enable the new cipher suites
			ss.setEnabledCipherSuites(suites);
			
	}
    
    
}

