//Listing 27.2 A simple polymorphic payroll example
/*
 * SimplePayroll2.java
 *
 */

package unleashed.ch27;
import java.util.*;

public class SimplePayroll2 {

    /** Creates new Payroll2 */
    public SimplePayroll2() {}
 
    public static void main(String[] args)
    {
        Company c1 = new Company(5000000, 1000000, 50);
        Vector s = new Vector();
        Employee e = null;
       
        s.addElement( new Executive(1, "Waller", "Marshall", 10000, .02)); 
        s.addElement( new SalariedEmployee(2, "Sanderson", "Martha", 4000)); 
        s.addElement( new HourlyEmployee(3, "Ethridge", "Bennett", 20.00f)); 
        s.addElement( new SalesEmployee(4, "Rangitsch", "Thomas", .0015)); 

        Iterator it = s.iterator();
        while (it.hasNext())
        {
            e = (Employee)it.next();
            System.out.println("Pay to the order of " + e.getFirstName() + " " + e.getLastName() + " " + e.calculateCheckAmount());
        }
    }
}

class Company
{
    private static long sales;
    private static long profit;
    private static int hours;
    
    public Company( long sales, long profit, int hours)
    {
        this.sales = sales;
        this.profit = profit;
        this.hours = hours;
    }
    
    public static long getSales()
    {
        return sales;
    }    
    
    public static long getProfit()
    {
        return profit;
    }

    public static int getHours()
    {
        return hours;
    }
}

abstract class Employee
{
    private int employeeID;
    private String lastName;
    private String firstName;
    
    public Employee(int employeeID, String lastName, String firstName)
    {
        this.employeeID = employeeID;
        this.lastName = lastName;
        this.firstName = firstName;
    }
    
    public String getFirstName()
    {
        return this.firstName;
    }
    
    public String getLastName()
    {
        return this.lastName;
    }
    
    abstract public double calculateCheckAmount();
}

class Executive extends Employee
{
    private int baseSalary;
    private double bonusPercentage;
    
    public Executive(int employeeID, String lastName, String firstName, int baseSalary, double bonusPercentage)
    {
        super(employeeID, lastName, firstName);
        this.baseSalary = baseSalary;
        this.bonusPercentage = bonusPercentage;
    }
    
    public double calculateCheckAmount()
    {
        double bonusAmt = Company.getProfit()*bonusPercentage;
        return baseSalary + bonusAmt;
    }
}

class SalariedEmployee extends Employee
{
    private int baseSalary;
    
    public SalariedEmployee(int employeeID, String lastName, String firstName, int baseSalary)
    {
        super(employeeID, lastName, firstName);
        this.baseSalary = baseSalary;
    }
    
    public double calculateCheckAmount()
    {
        return baseSalary;
    }
}

class HourlyEmployee extends Employee
{
    private float hourlyRate;
    
    public HourlyEmployee(int employeeID, String lastName, String firstName, float hourlyRate)
    {
        super(employeeID, lastName, firstName);
        this.hourlyRate = hourlyRate;
    }
    
    public double calculateCheckAmount()
    {
        int hours = Company.getHours();
        if ( hours < 40 )
        {
                return hours*hourlyRate;
        }
        else
        {
                return 40*hourlyRate + (hours-40)*hourlyRate*1.5;
        }
    }
}

